#ProcessorServer.py
'''Functions to communicate with server sites for download and upload.'''

import logging
import tarfile
from pathlib import Path
from string import Template

from iris import load
from iris.cube import CubeList

from coordinator.ProcessorUtils import subprocess_and_log, remove_path_from_tar_members


logger = logging.getLogger(__name__)


def get_data_from_server(jobPath,config,component):

    logger.info('Copying file from remote server to job directory')

    # Initialise output variable
    proc_out = {}
    # Output files available for upload    
    proc_out['output'] = None
    # Processing files available for clearing
    proc_out['clearup'] = []

    file_path = Template(config[component]['ServerPathTemplate']).substitute(**config)
    file_name = Template(config[component]['InputFileTemplate']).substitute(**config)

    #TODO: check if file exists already (may be the case for multiple configs in one)

    # TODO: perform ssh file transfer in python instead of subprocess
    server_name: str = config['ServerName']
    if server_name == "":
        cmd_scp: list = ["scp", f"{file_path}/{file_name}.tar.gz", jobPath]
    else:
        cmd_scp: list = ["scp", "-i", config['ServerKey'], "-o", "StrictHostKeyChecking=no",
                            f"{server_name}:{file_path}/{file_name}.tar.gz", jobPath]

    description_short = 'env2 scp'
    description_long = 'Copying file from remote server to job directory'
    # lawrence comment in/out
    subprocess_and_log(cmd_scp,description_short, description_long)

    logger.info('untarring the input file')

    # untar incoming name data
    output_directory = f"{jobPath}/NAME_Met_as_netcdf"
    Path(output_directory).mkdir(parents=True, exist_ok=True)
    tarfile_name = f"{jobPath}/{file_name}.tar.gz"
    with tarfile.open(tarfile_name) as tar:
        members = remove_path_from_tar_members(tar)
        tar.extractall(output_directory, members = members)

    # basic check that contents are as expected for 7-day forecast (57 timepoints in all files)
    cube_wildcard = f"{output_directory}/*.nc"
    cubes: CubeList = load(cube_wildcard)

    # land_fraction and topography will only have a single timepoint (as these dont change over time), so we can ignore
    # these when sense-checking the expected number of timepoints
    ignore_list = ["LAND_FRACTION", "TOPOGRAPHY"]

    for cube in cubes:
        var_name = cube.name()
        coord = cube.coord("time")
        timepoint_count = coord.shape[0]
        if timepoint_count != 57 and var_name not in ignore_list:
            msg = f"Unexpected number of timepoints ({timepoint_count}) in cube {cube.name()}"
            logger.error(msg)
            raise RuntimeError(msg)
        
    # tar file can be removed if required
    proc_out['clearup'] += [f"{jobPath}/{file_name}.tar.gz",]

    return proc_out
